<?php

// Helper method to get a string description for an HTTP status code
// From http://www.gen-x-design.com/archives/create-a-rest-api-with-php/ 
function getStatusCodeMessage($status)
{
    // these could be stored in a .ini file and loaded
    // via parse_ini_file()... however, this will suffice
    // for an example
    $codes = Array(
        100 => 'Continue',
        101 => 'Switching Protocols',
        200 => 'OK',
        201 => 'Created',
        202 => 'Accepted',
        203 => 'Non-Authoritative Information',
        204 => 'No Content',
        205 => 'Reset Content',
        206 => 'Partial Content',
        300 => 'Multiple Choices',
        301 => 'Moved Permanently',
        302 => 'Found',
        303 => 'See Other',
        304 => 'Not Modified',
        305 => 'Use Proxy',
        306 => '(Unused)',
        307 => 'Temporary Redirect',
        400 => 'Bad Request',
        401 => 'Unauthorized',
        402 => 'Payment Required',
        403 => 'Forbidden',
        404 => 'Not Found',
        405 => 'Method Not Allowed',
        406 => 'Not Acceptable',
        407 => 'Proxy Authentication Required',
        408 => 'Request Timeout',
        409 => 'Conflict',
        410 => 'Gone',
        411 => 'Length Required',
        412 => 'Precondition Failed',
        413 => 'Request Entity Too Large',
        414 => 'Request-URI Too Long',
        415 => 'Unsupported Media Type',
        416 => 'Requested Range Not Satisfiable',
        417 => 'Expectation Failed',
        500 => 'Internal Server Error',
        501 => 'Not Implemented',
        502 => 'Bad Gateway',
        503 => 'Service Unavailable',
        504 => 'Gateway Timeout',
        505 => 'HTTP Version Not Supported'
    );
 
    return (isset($codes[$status])) ? $codes[$status] : '';
}

//------------------------------------------------------------------------------

// Helper method to send a HTTP response code/message
function sendResponse($status = 200, $body = '', $content_type = 'text/html')
{
    $status_header = 'HTTP/1.1 ' . $status . ' ' . getStatusCodeMessage($status);
    header($status_header);
    header('Content-type: ' . $content_type);
    echo $body;
}

//------------------------------------------------------------------------------

class PLSAPI {
    private $db;

    // Constructor - open DB connection
    function __construct() {
        $this->db = new mysqli('ephesus.cs.cf.ac.uk', 'c1031804', 'plstestdb', 'c1031804');
        $this->db->autocommit(FALSE);
    }
 
    // Destructor - close DB connection
    function __destruct() {
        $this->db->close();
    }

//------------------------------------------------------------------------------

    function login($username) {
        $stmt = $this->db->prepare('SELECT pw_hash, pw_salt FROM engineer WHERE username=?');
        $stmt->bind_param("s", $username);
        $stmt->execute();
        $stmt->bind_result($pw_hash, $pw_salt);
        while ($stmt->fetch()) {
            break;
        }
        $stmt->close();
 
        // Bail if username doesn't exist (for security reasons we do not tell the user this)
        if ($pw_hash == "") {
            sendResponse(400, 'Invalid username or password');
            return false;
        }
  
        $result = array(
            "pw_hash" => $pw_hash,
            "pw_salt" => $pw_salt
        );

        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------

    function jobs($username, $status) {
        $stmt = $this->db->prepare('SELECT id, description, type FROM job WHERE engineer_id=(SELECT engineer_id FROM engineer WHERE username=?) AND status=?');
        $stmt->bind_param("si", $username, $status);
        $stmt->execute();
        $stmt->bind_result($id, $description, $type);
 
        $result = array();
        while ($stmt->fetch()) {
            $row = array(
                "id" => $id,
                "description" => $description,
                "type" => $type
            );
            $result[$id] = $row;
        }
        $stmt->close();

        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------

    function part($part_id) {
        $stmt = $this->db->prepare('SELECT id, make, model, description FROM part WHERE id=?');

        $stmt->bind_param("i", $part_id);
        $stmt->execute();

        $stmt->bind_result( $id,
                            $make,
                            $model,
                            $description );
 
        while ($stmt->fetch()) {
                break;
        }

        $stmt->close();

        // Part doesn't exist so respond with an error
        if ($make == "") {
            sendResponse(400, 'Part id <' . $part_id . '> does not exist');
            return false;
        }

        $result = array(
            "id" => $id,
            "make" => $make,
            "model" => $model,
            "description" => $description
        );

        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------

    function job_parts($job_id) {
        $stmt = $this->db->prepare('SELECT part_id, quantity FROM job_parts WHERE job_id=?');

        $stmt->bind_param("i", $job_id);
        $stmt->execute();

        $stmt->bind_result( $part_id,
                            $quantity );

        $index = 0;
        $result = array();
        while ($stmt->fetch()) {
            $row = array(
                "part_id" => $part_id,
                "quantity" => $quantity
            );

            $result[$index] = $row;
            ++$index;
        }
        $stmt->close();
 
        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------

    function add_job_part($job_id, $part_id, $quantity) {
        // First check the part exists at all
        $stmt = $this->db->prepare('SELECT id, make, model, description FROM part WHERE id=?');
        $stmt->bind_param("i", $part_id);
        $stmt->execute();
        
        $stmt->bind_result( $id,
                            $make,
                            $model,
                            $description );

        $check = false; 
        while ($stmt->fetch()) {
            $check = true;
            break;
        }

        $stmt->close();

        // Part does not exist so respond accordingly
        if (!$check) {
            sendResponse(400, 'Part ID <' . $part_id . '> does not exist');
            return false;
        }

        $result = array(
            "id" => $id,
            "make" => $make,
            "model" => $model,
            "description" => $description
        );

        // Now check if the part has been added to the job already, if so, just increase
        // the quantity otherwise we will end up violating our primary key constraint

        $stmt = $this->db->prepare('SELECT 1 FROM job_parts WHERE job_id=? AND part_id=?');
        
        $stmt->bind_param("ii", $job_id, $part_id);
        $stmt->execute();

        $check = false; 
        while ($stmt->fetch()) {
            $check = true;
            break;
        }

        $stmt->close();

        if ($check) {
            $stmt = $this->db->prepare('UPDATE job_parts SET quantity = quantity + ? WHERE job_id=? AND part_id=?');

            $stmt->bind_param("iii", $quantity, $job_id, $part_id);
            $stmt->execute();
            $stmt->close();

            sendResponse(200, json_encode($result));
            return true;
        }

        // Part has been added for this first time so do a normal insert
        $stmt = $this->db->prepare('INSERT INTO job_parts VALUES(?, ?, ?)');
        
        $stmt->bind_param("iii", $job_id, $part_id, $quantity);
        
        if (!$stmt->execute()) {
            sendResponse(400, 'Error: Failure whilst adding part to job');
            return false;
        }

        $stmt->close();

        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------
    
    function delete_job_part($job_id, $part_id) {
        $stmt = $this->db->prepare('DELETE FROM job_parts WHERE job_id=? AND part_id=?');

        $stmt->bind_param("ii", $job_id, $part_id);
        
        if (!$stmt->execute()) {
            sendResponse(400, 'Item Deletion Failed');
            return false;
        }

        $stmt->close();
 
        sendResponse(200, 'Item Deleted Successfully');
        return true;
    }

//------------------------------------------------------------------------------

    function update_job($job_id, $status, $closing_report) {
        $stmt = $this->db->prepare('UPDATE job SET status=?, closing_report=? WHERE id=?');
            
        $stmt->bind_param("isi", $status, $closing_report, $job_id);
        
        if (!$stmt->execute()) {
            $stmt->close();
            sendResponse(400, 'Error Updating Job');
            return false;
        }

        $stmt->close();
 
        sendResponse(200, 'Job Updated Successfully');
        return true;
    }

//------------------------------------------------------------------------------

    function atm($atm_id) {
        $stmt = $this->db->prepare('SELECT make, model, notes FROM atm WHERE id=?');

        $stmt->bind_param("i", $atm_id);
        $stmt->execute();

        $stmt->bind_result( $make,
                            $model,
                            $notes );
 
        while ($stmt->fetch()) {
                break;
        }

        $stmt->close();

        // Site doesn't exist so respond with an error
        if ($make == "") {
            sendResponse(400, 'Error: ATM id "' . $atm_id . '" does not exist');
            return false;
        }

        $result = array(
            "make" => $make,
            "model" => $model,
            "notes" => $notes
        );

        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------

    function site($site_id) {
        $stmt = $this->db->prepare('SELECT  name,
                                            addressLine1,
                                            addressLine2,
                                            town,
                                            county,
                                            postcode,
                                            phone
                                     FROM   site
                                     WHERE id=?');

        $stmt->bind_param("i", $site_id);
        $stmt->execute();

        $stmt->bind_result( $name,
                            $addressLine1,
                            $addressLine2,
                            $town,
                            $county,
                            $postcode,
                            $phone );
 
        while ($stmt->fetch()) {
                break;
        }

        $stmt->close();

        // Site doesn't exist so respond with an error
        if ($name == "") {
            sendResponse(400, 'Error: Site id "' . $site_id . '" does not exist');
            return false;
        }

        $result = array(
            "name" => $name,
            "addressLine1" => $addressLine1,
            "addressLine2" => $addressLine2,
            "town" => $town,
            "county" => $county,
            "postcode" => $postcode,
            "phone" => $phone
        );

        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------

    function job($job_id) {
        $stmt = $this->db->prepare('SELECT  j.description,
                                            s.id,
                                            s.name,
                                            a.id,
                                            CONCAT(a.make, " ", a.model),
                                            j.type,
                                            j.details,
                                            j.parking
                                     FROM   job AS j,
                                            site AS s,
                                            atm AS a
                                     WHERE  j.id=? AND
                                            s.id = j.site_id AND
                                            a.id = j.atm_id');

        $stmt->bind_param("i", $job_id);
        $stmt->execute();

        $stmt->bind_result( $description,
                            $site_id,
                            $site_name,
                            $atm_id,
                            $atm_name,
                            $job_type,
                            $job_details,
                            $parking );
 
        while ($stmt->fetch()) {
                break;
        }

        $stmt->close();

        // Job doesn't exist so respond with an error
        if ($description == "") {
            sendResponse(400, 'Error: Job id "' . $job_id . '" does not exist');
            return false;
        }

        $stmt = $this->db->prepare('SELECT  jp.part_id,
                                            jp.quantity,
                                            p.make,
                                            p.model 
                                    FROM
                                            job_parts AS jp,
                                            part AS p
                                    WHERE
                                            p.id = jp.part_id AND
                                            jp.job_id=?');

        $stmt->bind_param("i", $job_id);
        $stmt->execute();

        $stmt->bind_result( $part_id,
                            $quantity,
                            $make,
                            $model );
 
        $index = 0;
        $parts = array();
        while ($stmt->fetch()) {
                $part = array(
                    "part_id" => $part_id,
                    "quantity" => $quantity,
                    "make" => $make,
                    "model" => $model
                );

                $parts[$index] = $part;
                ++$index;
        }

        $stmt->close();
        $result = array(
            "description" => $description,
            "site_id" => $site_id,
            "site_name" => $site_name,
            "atm_id" => $atm_id,
            "atm_name" => $atm_name,
            "job_type" => $job_type,
            "job_details" => $job_details,
            "parking" => $parking,
            "parts" => $parts
        );

        sendResponse(200, json_encode($result));
        return true;
    }

//------------------------------------------------------------------------------

    function execute() {
        if (isset($_POST["request"])) {

            $request = $_POST["request"];

            if ("login" == $request && isset($_POST["username"])) {
                return $this->login($_POST["username"]);
            }
            if ("jobs" == $request && isset($_POST["username"]) && isset($_POST["status"])) {
                return $this->jobs($_POST["username"], $_POST["status"]);
            }
            if ("job" == $request && isset($_POST["job_id"])) {
                return $this->job($_POST["job_id"]);
            }
            if ("site" == $request && isset($_POST["site_id"])) {
                return $this->site($_POST["site_id"]);
            }
            if ("atm" == $request && isset($_POST["atm_id"])) {
                return $this->atm($_POST["atm_id"]);
            }
            if ("part" == $request && isset($_POST["part_id"])) {
                return $this->part($_POST["part_id"]);
            }
            if ("job_parts" == $request && isset($_POST["job_id"])) {
                return $this->job_parts($_POST["job_id"]);
            }
            if ("add_job_part" == $request && isset($_POST["job_id"]) &&
                                              isset($_POST["part_id"]) &&
                                              isset($_POST["quantity"])) {

                return $this->add_job_part($_POST["job_id"],
                                           $_POST["part_id"],
                                           $_POST["quantity"]);
            }
            if ("delete_job_part" == $request && isset($_POST["job_id"]) &&
                                                 isset($_POST["part_id"])) {

                return $this->delete_job_part($_POST["job_id"], $_POST["part_id"]);
            }
            if ("update_job" == $request && isset($_POST["job_id"]) &&
                                            isset($_POST["status"]) &&
                                            isset($_POST["closing_report"])) {

                return $this->update_job($_POST["job_id"],
                                         $_POST["status"],
                                         $_POST["closing_report"]);
            }
        }

        sendResponse(400, 'Invalid request');
        return false;
    }
}

//------------------------------------------------------------------------------

$api = new PLSAPI;
$api->execute();
?>
